@@ -1,6 +1,3 @@
-# vi:fdm=marker fdl=0
-# $Id: SGF2misc.pm,v 1.7 2006/03/09 19:00:22 jettero Exp $ 
-
 package Games::Go::SGF2misc;
 
 use strict;
@@ -10,8 +7,9 @@ use Carp;
 use Parse::Lex;
 use Data::Dumper;
 use Compress::Zlib;
+use CGI qw(escapeHTML);
 
-our $VERSION = "0.9.7";
+our $VERSION = "0.9780";
 
 1;
 
@@ -41,6 +39,8 @@ sub parse {
         local $/;  # Enable local "slurp" ... ie, by unsetting $/ for this local scope, it will not end lines on \n
         open SGFIN, $file or die "couldn't open $file: $!";
 
+        our $FILENAME = $file;
+
         return $this->parse_internal(\*SGFIN);
     }
 
@@ -53,6 +53,8 @@ sub parse_string {
     my $this = shift;
     my $string = shift;
 
+    our $FILENAME = "STRING";
+
     return $this->parse_internal($string);
 }
 # }}}
@@ -61,6 +63,8 @@ sub parse_internal {
     my $this = shift;
     my $file = shift;
 
+    our $FILENAME;
+
     for my $k (keys %$this) {
         delete $this->{$k} unless {Time=>1, frm=>1}->{$k};
     }
@@ -71,14 +75,14 @@ sub parse_internal {
     my @rules = (
         VALUE  => '\[(?ms:.*?(?<!\x5c))\]',
 
-        BCOL   => '\(',         # begin collection
-        ECOL   => '\)',         # end collection
-        PID    => '[A-Z]+',     # property identifier
-        NODE   => ';',          # new node
+        BCOL   => '\(',                   # begin collection
+        ECOL   => '\)',                   # end collection
+        PID    => '(?:CoPyright|[A-Z]+)', # property identifier (CoPyright is the spurious IGS tag, assholes)
+        NODE   => ';',                    # new node
         WSPACE => '[\s\r\n]',
 
         qw(ERROR  .*), sub {
-            $global::lex_error = "Parse Error reading $file: $_[1]\n";
+            $global::lex_error = "Parse Error reading $FILENAME: $_[1]\n";
         }
     );
 
@@ -103,7 +107,7 @@ sub parse_internal {
             my $V = $token->text;
 
             if( $C eq "ERROR" or defined $global::lex_error ) {
-                $global::lex_error = "Parse Error reading $file: unknown";
+                $global::lex_error = "Parse Error reading $FILENAME: unknown"; # TODO: this $file should be ... the name of it instead
                 $this->{error} = $global::lex_error;
                 return 0;
             }
@@ -126,7 +130,7 @@ sub parse_internal {
             # this get's it's own if block for the $pid
             if( $C eq "PID" ) {
                 if( $nos == -1 ) {
-                    $this->{error} = "Parse Error reading $file: property identifier ($V) in strange place";
+                    $this->{error} = "Parse Error reading $FILENAME: property identifier ($V) in strange place";
                     return 0;
                 }
                 push @{ $ref->{n}[$nos] }, {P=>$V};
@@ -137,7 +141,7 @@ sub parse_internal {
                 $V =~ s/\\(.)/$1/msg;
 
                 if( $nos == -1 or $pid == -1 ) {
-                    $this->{error} = "Parse Error reading $file: property value ($V) in strange place";
+                    $this->{error} = "Parse Error reading $FILENAME: property value ($V) in strange place";
                     return 0;
                 }
                 if( defined $ref->{n}[$nos][$pid]{V} ) {
@@ -147,6 +151,9 @@ sub parse_internal {
 
                 $ref->{n}[$nos][$pid]{V} = $V;
 
+            } elsif( $C eq "WSPACE" ) {
+                # don't set pid to -1 here (2006-5-12)
+
             } else {
                 $pid = -1;
             }
@@ -386,88 +393,253 @@ sub as_text {
     return $x;
 }
 # }}}
+# _mark_alg {{{
+sub _mark_alg {
+    my $this = shift;
+    my ($mark, $img) = @_;
+
+    return "bt.gif" if $mark eq "TR" and $img eq "b.gif";
+    return "wt.gif" if $mark eq "TR" and $img eq "w.gif";
+    return "bc.gif" if $mark eq "CR" and $img eq "b.gif";
+    return "wc.gif" if $mark eq "CR" and $img eq "w.gif";
+    return "bq.gif" if $mark eq "SQ" and $img eq "b.gif";
+    return "wq.gif" if $mark eq "SQ" and $img eq "w.gif";
+
+    if( ($mark = int($mark)) > 0 and $mark <= 100 ) {
+        return "b$mark.gif" if $img eq "b.gif";
+        return "w$mark.gif"
+    }
+
+    return $img;
+}
+# }}}
+# _crazy_moku_alg {{{
+sub _crazy_moku_alg {
+    my $this = shift;
+    my ($i, $j, $size) = @_;
+
+    our $cma_size;
+    our $hoshi;
+
+    if( $size != $cma_size or not $hoshi ) {
+        $hoshi = {};
+        if( $size == 19 ) {
+            $hoshi = { "3 3" => 1, "3 15" => 1, "15 3" => 1, "15 15" => 1,
+                "9 3" => 1, "9 15" => 1, "3 9" => 1, "15 9" => 1, "9 9" => 1, };
+        } elsif( $size == 13 ) {
+            $hoshi = { "3 3" => 1, "9 9" => 1, "3 9" => 1, "9 3" => 1,
+                "6 3" => 1, "3 6" => 1, "9 6" => 1, "6 9" => 1, "6 6" => 1, };
+        } elsif( $size == 9 ) {
+            $hoshi = { "2 2" => 1, "2 6" => 1, "6 6" => 1, "6 2" => 1, "4 4" => 1, };
+        }
+    }
+
+    return "ulc.gif" if $i == 0     and $j == 0;
+    return "urc.gif" if $i == 0     and $j == $size;
+    return "llc.gif" if $i == $size and $j == 0;
+    return "lrc.gif" if $i == $size and $j == $size;
+    return "ts.gif"  if $i == 0     and $j != 0 and $j != $size;
+    return "bs.gif"  if $i == $size and $j != 0 and $j != $size;
+    return "ls.gif"  if $j == 0     and $i != 0 and $i != $size;
+    return "rs.gif"  if $j == $size and $i != 0 and $i != $size;
+
+    return "h.gif" if $hoshi->{"$i $j"};
+    return "p.gif",
+}
+# }}}
 # as_html {{{
 sub as_html {
-    my $this = shift;
-    my $node = shift;
-    my $dir  = shift;
-       $dir  = "./img" unless $dir;
+    my $this  = shift;
+    my $node  = shift;
+    my $dir   = shift;
+       $dir   = "./img" unless $dir;
+    my $id    = shift;
+    my $onode = $node;
 
     $node = $this->as_perl( $node, 1 ) or croak $this->errstr;
 
+    # use Data::Dumper; $Data::Dumper::Indent = 0;
+    # warn Dumper( $node );
+
+    my $gref      = $this->as_perl(1);
+    my $game_info = $gref->{game_properties};
+     
+=cut
+game_properties' => {'FF' => 4,'PB' => 'Orien Vandenbergh
+(nichus)','GM' => 1,'KM' => '6.5','SZ' => 19,'PC' => 'Dragon Go Server: http://www.dragongoserver.net','RE' => 'W+29.5','RU' =>
+'Japanese','BR' => '13 kyu','GN' => 'jettero-nichus-20041229.sgf','GC' => 'Game ID: 85389','DT' => '2004-10-29,2004-12-29','PW' =>
+'Jettero Heller (jettero)','WR' => '13 kyu','OT' => '30 days + 1 day/10 periods Japanese byoyomi'}
+=cut
+
     my $board = $node->{board};
     my $size  = @{$board->[0]}; # inaccurate?
-
-    my $hoshi = {};
-    if( $size == 19 ) {
-        $hoshi = { "3 3" => 1, "3 15" => 1, "15 3" => 1, "15 15" => 1,
-            "9 3" => 1, "9 15" => 1, "3 9" => 1, "15 9" => 1, "9 9" => 1, };
-    } elsif( $size == 13 ) {
-        $hoshi = { "3 3" => 1, "9 9" => 1, "3 9" => 1, "9 3" => 1,
-            "6 3" => 1, "3 6" => 1, "9 6" => 1, "6 9" => 1, "6 6" => 1, };
-    } elsif( $size == 9 ) {
-        $hoshi = { "2 2" => 1, "2 6" => 1, "6 6" => 1, "6 2" => 1, "4 4" => 1, };
-    }
-
-    $size--;
-
-    my $crazy_moku_alg = sub { my ($i, $j) = @_;
-        return "ulc.gif" if $i == 0     and $j == 0;
-        return "urc.gif" if $i == 0     and $j == $size;
-        return "llc.gif" if $i == $size and $j == 0;
-        return "lrc.gif" if $i == $size and $j == $size;
-        return "ts.gif"  if $i == 0     and $j != 0 and $j != $size;
-        return "bs.gif"  if $i == $size and $j != 0 and $j != $size;
-        return "ls.gif"  if $j == 0     and $i != 0 and $i != $size;
-        return "rs.gif"  if $j == $size and $i != 0 and $i != $size;
-
-        return "h.gif" if $hoshi->{"$i $j"};
-        return "p.gif",
-    };
+       $size--;
 
     my %marks = ();
     for my $m (@{ $node->{marks} }) {
         $marks{"$m->[1] $m->[2]"} = ($m->[0] eq "LB" ? $m->[4] : $m->[0]);
     }
 
-    my $mark_alg = sub { 
-        my ($mark, $img) = @_;
+    my $arow = "<tr align='center'><td>" . join("", map("<td>$_", qw(A B C D E F G H J K L M N O P Q R S T))) . "<td>";
 
-        return "bt.gif" if $mark eq "TR" and $img eq "b.gif";
-        return "wt.gif" if $mark eq "TR" and $img eq "w.gif";
-        return "bc.gif" if $mark eq "CR" and $img eq "b.gif";
-        return "wc.gif" if $mark eq "CR" and $img eq "w.gif";
-        return "bq.gif" if $mark eq "SQ" and $img eq "b.gif";
-        return "wq.gif" if $mark eq "SQ" and $img eq "w.gif";
+    my $x = "<table class='sgf2miscboard' cellpadding=0 cellspacing=0>$arow\n";
 
-        if( ($mark = int($mark)) > 0 and $mark <= 100 ) {
-            return "b$mark.gif" if $img eq "b.gif";
-            return "w$mark.gif"
-        }
-
-        return $img;
-    };
-
-    my $x = "<table cellpadding=0 cellspacing=0>\n";
     for my $i (0..$#{ $board }) {
-        $x .= "<tr>";
-        for my $j (0..$#{ $board->[$i] }) {
+        $x .= "<tr><td>". (19-$i);
+        for my $j (0 .. $#{ $board->[$i] }) {
+            my $iid = "";
+               $iid = " id='$id.$i.$j'" if $id;
 
             my $c = { 
                 'B' => "b.gif",
                 'W' => "w.gif",
             }->{$board->[$i][$j]};
 
-            $c = $crazy_moku_alg->($i, $j) unless $c;
-            $c = $mark_alg->($marks{"$i $j"}, $c);
+            $c = "wc.gif" if $c eq "w.gif" and $node->{moves}[0][1] == $i and $node->{moves}[0][2] == $j;
+            $c = "bc.gif" if $c eq "b.gif" and $node->{moves}[0][1] == $i and $node->{moves}[0][2] == $j;
 
-            $c  = "$dir/$c";
-            $x .= "<td><img src=\"$c\" width=19 height=19></td> ";
+            $c = $this->_crazy_moku_alg($i, $j, $size) unless $c;
+            $c = $this->_mark_alg($marks{"$i $j"}, $c);
+
+            $c  = "$dir/$c"; 
+            $x .= "<td><img$iid src=\"$c\">";
         }
-        $x .= "</tr>\n";
+        $x .= "<td align='right'>". (19-$i) . "\n";
     }
 
-    return "$x</table>";
+    my $cpid = "";
+       $cpid = " id='$id.state'" if $id;
+
+    my $p = "<tr><td$cpid colspan='21' class='sgf2misccaps'><br/>W.Caps: $node->{captures}->{W}\&nbsp; \&nbsp;B.Caps: $node->{captures}->{B}";
+
+    if( $node->{other} ) {
+        my $TB = $node->{other}->{TB};
+        my $TW = $node->{other}->{TW};
+
+        if ($TB and $TW) {
+            my $cb = $node->{captures}->{B};
+            my $cw = $node->{captures}->{W};
+            my $km = $game_info->{KM};
+
+            my ($tb, $tw) = (0, 0);
+            for my $r (@$TB) {
+                my @a = $this->sgfco2numco($gref, $r);
+
+                $tb ++;
+                $cb ++ if $board->[$a[0]][$a[1]] eq "W";
+            }
+
+            for my $r (@$TW) {
+                my @a = $this->sgfco2numco($gref, $r);
+
+                $tw ++;
+                $cw ++ if $board->[$a[0]][$a[1]] eq "B";
+            }
+
+            my $f = ($tw + $cw + $km) - ($tb + $cb);
+               $f = ($f<0 ? "B+".abs($f) : "W+$f");
+
+            $p = "<tr><td$cpid colspan='21' class='sgf2miscresult'><br/>W($tw t + $cw c + $km k), B($tb t + $cb c): $f";
+        }
+    }
+
+    my $cmid = "";
+       $cmid = " id='$id.comment'" if $id;
+
+    my $comments = "";
+       $comments .= escapeHTML($_) for @{$node->{comments}};
+       $comments =~ s/[\r\n]/<br\/>/sg;
+
+    return "$x$arow$p</table><!--MATCHME--><div$cmid class='sgf2misccomment'>$comments</div>";
+}
+# }}}
+# as_js {{{
+sub as_js {
+    my $this  = shift;
+    my $node  = shift;
+
+    $node = $this->as_perl( $node, 1 ) or croak $this->errstr;
+
+    my $gref      = $this->as_perl(1);
+    my $game_info = $gref->{game_properties};
+
+    my $board = $node->{board};
+    my $size  = @{$board->[0]}; # inaccurate?
+       $size--;
+
+    my %marks = ();
+    for my $m (@{ $node->{marks} }) {
+        $marks{"$m->[1] $m->[2]"} = ($m->[0] eq "LB" ? $m->[4] : $m->[0]);
+    }
+
+    my @board  = ();
+    for my $i (0..$#{ $board }) {
+        my $row = [];
+        for my $j (0 .. $#{ $board->[$i] }) {
+            my $c = { 
+                'B' => "b.gif",
+                'W' => "w.gif",
+            }->{$board->[$i][$j]};
+
+            $c = "wc.gif" if $c eq "w.gif" and $node->{moves}[0][1] == $i and $node->{moves}[0][2] == $j;
+            $c = "bc.gif" if $c eq "b.gif" and $node->{moves}[0][1] == $i and $node->{moves}[0][2] == $j;
+
+            $c = $this->_crazy_moku_alg($i, $j, $size) unless $c;
+            $c = $this->_mark_alg($marks{"$i $j"}, $c);
+
+            push @$row, $c;
+        }
+
+        push @board, $row;
+    }
+
+    local $Data::Dumper::Indent = 0;
+    my $board = Dumper(\@board);
+       $board =~ s/^\$VAR1\s*=\s*//s;
+       $board =~ s/\s*\;\s*$//s;
+       $board =~ s/\.gif//sg;
+
+    my $p = "<br/>W.Caps: $node->{captures}->{W}\&nbsp; \&nbsp;B.Caps: $node->{captures}->{B}";
+
+    if( $node->{other} ) {
+        my $TB = $node->{other}->{TB};
+        my $TW = $node->{other}->{TW};
+
+        if ($TB and $TW) {
+            my $cb = $node->{captures}->{B};
+            my $cw = $node->{captures}->{W};
+            my $km = $game_info->{KM};
+
+            my ($tb, $tw) = (0, 0);
+            for my $r (@$TB) {
+                my @a = $this->sgfco2numco($gref, $r);
+
+                $tb ++;
+                $cb ++ if $board->[$a[0]][$a[1]] eq "W";
+            }
+
+            for my $r (@$TW) {
+                my @a = $this->sgfco2numco($gref, $r);
+
+                $tw ++;
+                $cw ++ if $board->[$a[0]][$a[1]] eq "B";
+            }
+
+            my $f = ($tw + $cw + $km) - ($tb + $cb);
+               $f = ($f<0 ? "B+".abs($f) : "W+$f");
+
+            $p = "<br/>W($tw t + $cw c + $km k), B($tb t + $cb c): $f";
+        }
+    }
+
+    my $comments = "";
+       $comments .= escapeHTML($_) for @{$node->{comments}};
+       $comments =~ s/[\r\n]/<br\/>/sg;
+
+           $p =~ s/"/\\"/sg;
+    $comments =~ s/"/\\"/sg;
+
+    return "{ board: $board, status: \"$p\", comment: \"$comments\" }";
 }
 # }}}
 # as_image {{{
@@ -705,6 +877,9 @@ sub _parse {
             if( $p->{P} =~ m/$gm_pr_reg/ ) {
                 $gref->{game_properties}{$p->{P}} = $p->{V};
             }
+            if( $p->{P} eq "CoPyright" ) {
+                $gref->{game_properties}{FF} = 4;
+            }
         }
 
         unless( $gref->{game_properties}{GM} == 1 ) {
@@ -1065,268 +1240,3 @@ sub _find_group {
 }
 # }}}
 
-__END__
-# Below is stub documentation for your module. You better edit it!
-
-=head1 NAME
-
-    Games::Go::SGF2misc - Reads SGF files and produces usable output in many formats
-
-=head1 SYNOPSIS
-
-    use Games::Go::SGF2misc;
-
-    my $sgf = new Games::Go::SGF2misc;
-    for my $file (<*.sgf>) {
-        $sgf->parse($file);
-
-        # do things with the parsed sgf:
-        # the output options are listed below
-    }
-
-=head1 nodelist
-
-    my $nodelist = $sgf->nodelist;  
-
-    # This returns a special list of node-id's (human readable
-    # $game_number.$variation-$move_no node descriptors).  example:
-
-    $nodelist = { 1 => [
-        [ '1.1-root', '1.1-1', '1.1-2', '1.1-3', '1.1-4', '1.1-5', ],
-        [      undef,   undef,   undef,   undef,   undef, '1.2-5', '1.2-6', '1.2-7', ],
-        [      undef,   undef,   undef,   undef,   undef,   undef,   undef, '1.3-7', '1.3-8' ],
-    ] };
-
-    # What do you do with them?
-
-=head1 as_perl
-
-    my $game_info = $sgf->as_perl(1);
-    my $root_node = $sgf->as_perl('1.1-root');
-    my $move5_v2  = $sgf->as_perl('1.2-5');
-
-    my $s = $game_info->{game_properties}{SZ};
-    print "The board size is: ${s}x${s}!\n"; 
-
-    print my $c (@{ $move5_v2->{comments} }) {
-        print "Comment from 1.2-5: $c\n";
-    }
-
-    # Tada!!
-
-    # Oh, as_perl takes an optional second argument.
-    
-    $sgf->as_perl("doesn't exist node"); # will die on the spot...
-    $sgf->as_perl("isn't there", 1)      # will not
-        or print "yikes!!: " . $sgf->errstr;
-
-=head1 as_text 
-
-    # This is pretty much just an example
-
-    use strict;
-    use Games::Go::SGF2misc;
-
-    my $sgf = new Games::Go::SGF2misc; 
-       $sgf->parse("sgf/jettero-sixrusses-2004-03-18.sgf");
-
-    my $nl  = $sgf->nodelist;
-    my $end = $nl->{1}[0][ $#{$nl->{1}[0]} ];
-              # 1st game  1st variation    last node
-     
-    my $caps = $sgf->as_perl( $end )->{captures};
-
-    print $sgf->as_text($end), "Captures:  Black-$caps->{B} / White-$caps->{W}\n";
-
-    # Result:
-    #   X O . . . O . . . . . . . . . O O O .
-    #   . O O . O O X . X X O O X O O O X X X
-    #   X O . . . O X . X O . O O . O X . . .
-    #   O O . O O X O O O O O . . O X . X . .
-    #   O O O O O X X X X O . O O X . . . . .
-    #   O X X X O X . X . X O O X . X . . . .
-    #   X X X O O . X . X . X X . . . . . . .
-    #   X . . X O . . X . . . . . . . . X . .
-    #   X . X X O O O X . . . . . . . . . . .
-    #   X . X . X X O X O O O X . . . . . . .
-    #   . X . . . X O O X X X X X . . . X . .
-    #   . . . . X O O . O O O X . . . X X X X
-    #   X X X . X X O O . O X X X . . X O O X
-    #   X O O X . . X . O O O X X . X O O . O
-    #   O O . O X X X X X O X X O X X X O . .
-    #   . . O O O X . . X O O X O X O X O X .
-    #   . . . . O O X X O O . O O O O O O X .
-    #   . . . . . O X O . O . . . . O X X O O
-    #   . . . . . O X O O . . . . . . . . . .
-    #   Captures:  Black-11 / White-16
-
-=head1 as_html 
-
-    # This function works very much like the as_text function above, but
-    # instead prints out an html table full of images.
-
-    open OUT, ">example.html" or die $!;
-    print OUT, $sgf->as_html( $end, "/image/dir/" );
-
-    # The only real difference is the image-dir argument (which defaults to
-    # "./img").  This is the URL image dir (<img # src="$image_dir/moku.gif">), 
-    # not the pysical image dir.  There is a directory of images included
-    # with SGF2misc.  They are from this page:
-
-    # http://www.britgo.org/gopcres/gopcres1.html#ag-gifs
-
-    # They are Andrew Grant's GIF images.  I did NOT seek permission to
-    # re-distribute them.  Perhaps I have no right to do so.  I really
-    # don't know how to get ahold of him.  
-
-    # If anyone knows how who to ask, please tell me.  If anyone knows it's
-    # a problem, please tell me.
-
-    # 3/22/04, Orien Vandenbergh made bc.gif, wc.gif, bq.gif and wq.gif for me.
-
-    # NOTE: On marks, this as_html only shows circles, triangles, squares,
-    # and numbers where there are stones.  It does not show letters at all.
-    # This is only because I don't have images for _everything_. :)
-
-=head1 as_image
-
-    # This uses the fantastic ::SGF2misc::GD package by Orien Vandenbergh
-    # that comes with this package.  It is a separate package, and as you
-    # will see, the interfaces aren't totally compatable -- however, it
-    # _is_ intended to be used with this package.
-
-    # You must install GD-2.15 (or so) in order to use it!!  You will also
-    # need the bleeding edge versions of libpng and libgd.  At the time of
-    # this writing, I used libgd-2.0.22 and got GD-2.12 to install and
-    # function normally.
-
-    # Here's a calling example:
-    $sgf->as_image($node, {filename=>"html/$x->[$i].png", gobanColor=>[255, 255, 255]});
-
-    # ::SGF2misc::GD takes hash-like arguments.  So, so does as_image().
-    # filename=>"" and gobanColor=>[] are additions of mine, as they're
-    # actually used on separate calls in the ::SGF2misc::GD package.
-    # All other arguments are passed to the new() member function if
-    # ::SGF2misc::GD.  Please read the Games::Go::SGF2misc::GD manpage; it
-    # contains much more information.
-
-    # Some SGF writers will automatically add a CR (circle) property to
-    # the current move, which as_image will render as expected.  However
-    # other SGF writers do not.  To have as_image automatically render
-    # circles on the current moves add the hash argument auto_mark=>1.
-    # With auto_mark enabled, any nodes which already have _any_ form of
-    # markup will not receive _any_ circles on the current moves.  This
-    # is to help ensure consistancy with SGFs where the markup has other
-    # (more important?) uses.*This could have undesirable results for
-    # nodes in which there are multiple stones placed.
-
-    # Also, in the interests of making this as clumsy as possible, if the
-    # filename is a dash followed by a type extension,
-
-        my $image = $sgf->as_image($l, {filename=>"-.png"});
-
-    # then the image will be returned as a string rather than written to a
-    # file.  
-
-    # Optionally, you can also install the ::SGF2misc::SVG package written
-    # by Orien.  To specify use of the SVG module call as_image with the 
-    # argument 'use' => 'Games::Go::SGF2misc::SVG'.  Functionally the two 
-    # rendering modules are equivalent, except SVG doesn't yet support
-    # returning the image as a string.  Aesthetically, however, ::SVG
-    # seems to render a cleaner image (at all resolutions), and does so
-    # significantly faster.
-
-=head1 as_freezerbag
-  
-   # What?  Yeah, this is a special way to save the parsed sgf data such
-   # that you can interact with it without having to re-parse it.  It
-   # seemed like it should be faster than re-parsing the SGF every time,
-   # but it isn't much of a speedup at all.  Unless I can speed it up, or
-   # someone asks me to leave it in, it could get axed.  I may leave it in
-   # anyway, because it's cute.
-
-   # Try it yourself:
-
-    $sgf->as_freezerbag( "freezer.pl",  # an export filename
-
-        # Some code to put in it.  This argument is optional.
-            q/my @nodes = @{ $sgf->nodelist->{1}[0] };
-
-            for my $n (@nodes) {
-                my $board = $sgf->as_text($n) or die $sgf->errstr; 
-                print "\e[1;1f$board";
-            }/,
-
-       # The location and/or switches for your perl
-       # I needed blib/lib (since I don't always install to test this
-       # stuff).  This argument is optional.
-
-       "/usr/bin/perl -I blib/lib",
-    );
-
-=head1 Board Postion Character Map
-
-    # This is how an empty 3x3 board would be stored:
-
-    $board = [
-        [ ' ', ' ', ' ' ],
-        [ ' ', ' ', ' ' ],
-        [ ' ', ' ', ' ' ],
-    ];
-
-    # ' ' - an empty board position
-    # 'W' - a white stone
-    # 'B' - a black stone
-
-    # Marks are not placed on the board!
-    # You'll just have to fetch the marks array from the $node.
-
-=head1 Miscellaneous
-
-=head2 is_node
-
-    print "There is a node called 1.1-root!\n" if $sgf->is_node('1.1-root');
-
-=head1 BUGS
-
-    Besides the lack of documentation?  Well, I'm sure there's a bunch.
-    If you spot any bugs, please tell me.
-
-=head1 ENV VARIABLES
-
-=head2 DEBUG
-
-   Try setting $ENV{DEBUG}=1, $ENV{DEBUG}=2, or $ENV{DEBUG}=3 to see the internals.
-
-   Also, from your bash prompt you can 'DEBUG=1 perl ./myprog.pl' to
-   enable debugging dynamically.
-
-   DEBUG of 31 or over will show the lexical trace.  That's kinda fun.
-
-=head1 AUTHOR
-
-    Please contact me with ANY suggestions, no matter how pedantic.
-
-    Jettero Heller <jettero@cpan.org>
-
-    Some changes and patches provided by:
-
-    Orien Vandenbergh <orien@icecode.com>
-
-=head1 COPYRIGHT
-
-    GPL!  I included a gpl.txt for your reading enjoyment.
-
-    Though, additionally, I will say that I'll be tickled if you were to
-    include this package in any commercial endeavor.  Also, any thoughts to
-    the effect that using this module will somehow make your commercial
-    package GPL should be washed away.
-
-    I hereby release you from any such silly conditions.
-
-    This package and any modifications you make to it must remain GPL.  Any
-    programs you (or your company) write shall remain yours (and under
-    whatever copyright you choose) even if you use this package's intended
-    and/or exported interfaces in them.
-
-=cut
